/*******************************************************************************
*
* Motorola Inc.
* (c) Copyright 2003 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
********************************************************************************
*
* FILE NAME: bootloader.c
*
*******************************************************************************/

#include "bsp.h"
#include "io.h"
#include "periph.h"
#include "appconfig.h" 
#include "bootloader.h"
#include "pll.h"
#include "flashdrv.h"
#include "timer.h"
#include "highlevelmapping.h"

#ifdef INCLUDE_SCI 
	#include "sci.h"
	#include "scidrv.h"
#endif 

#ifdef INCLUDE_CAN 
	#include "can.h"
	#include "flexcan.h"
#endif

/****************************************************************************/
// global variables
/****************************************************************************/
handle_t  DataFlashFD;
handle_t  ProgFlashFD;
UWord32   StartAddress;
UWord32   DataCounter = 0;
UWord32   NewStartAddress;
UWord32   InterruptVectorAddress;
UWord32   NewInterruptVectorAddress;
UInt8     StrBuffer[12];
UWord16   Temp;
UInt8     SRecordChecksum;
handle_t  DriverFD  = IO_INVALID_HANDLE;
bool      DataInRAM = false;

/****************************************************************************
/ constants
/****************************************************************************/
const unsigned char StrWaitingSrec[]     = "\r\n\r\n Waiting for application S-Record.";
const unsigned char StrLoaded_1[]        = ".";
const unsigned char StrLoaded_2[]        = "\r\n Loaded 0x";
const unsigned char StrLoaded_3[]        = " bytes.";
const unsigned char StrStarted_1[]       = "\r\n Application started from address 0x";
const unsigned char StrError_1[]         = "\r\n Error # ";
const unsigned char HexTable[16]         = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
const unsigned char StrCopyright_1[]     = "\r\n\r\n (c) 2003 MOTOROLA. S-Record loader for the MC56F83xx. ver. 1.0.3";

/***************************************************************************
/ extern variables
****************************************************************************/
extern UWord16             RegState[];
extern arch_PeriphRegState peripheralReg[archoff_registers_num];

extern void * _vector_begin_addr;   
extern void * _Xdata_begin_addr_in_RAM;
extern void * _Xdata_end_addr_in_RAM;
extern void * _Pcode_begin_addr_in_ROM;
extern void * _Pcode_end_addr_in_ROM;
extern void * _memEXbit;

/**************************************************************************
* Functions:   Main()
*
* Description: SCI and CAN bootloader
*
* Returns:     None
*
* Notes:       None 
*
**************************************************************************/
void main(void)
{
	UWord16 i;
	UWord16 StartDelay;	
	UWord16 DataLength; 
	UInt8   CharBuffer[10];
	UInt8   DataBuffer[525];
	UWord32 DataAddress;
	bool    bootVersionPrompt;
	bool    bootContinueFlag  = false;
	bool    bootXDATAFlag     = false;
	
	
  /****************************************************************************
  / read user start address 
  /****************************************************************************/
	StartAddress = *((UWord32 *)BOOT_START_ADDRESS_PLACE);

  /****************************************************************************
  / read start delay if no delay jump to user application
  /****************************************************************************/
    StartDelay = (*((UWord16 *)BOOT_START_DELAY_PLACE)) & 0x00FF;

    if(StartDelay == 0)
    {
		bootExit();
    }
    
  /****************************************************************************
  / check to see if version prompt should be enabled
  /****************************************************************************/
	if(((*((UWord16 *)BOOT_VERSION_PROMPT_PLACE)) >> 8) == 0)
    { 
		bootVersionPrompt = false;
	}
	else
	{
		bootVersionPrompt = true;
	}
	
  /****************************************************************************
  / set application start address to default if none defined
  /****************************************************************************/
	if(StartAddress == 0xFFFFFFFF)
	{
		StartAddress = BOOT_DEFAULT_APP_START_ADDR;
	}

  /****************************************************************************
  / Configure interrupt vector base address to P:0x20000
  /****************************************************************************/
	NewInterruptVectorAddress = (UWord32)&_vector_begin_addr;
	periphMemWrite((NewInterruptVectorAddress >> 8), &ArchIO.IntCtrl.VectorBaseAddrReg);
 
  /****************************************************************************
  /  OPEN PERIPHERAL DRIVER THAT WILL BE USED TO LOAD S-RECORD FILE: see
  /  highlevelmapping.h for details on how specific peripheral drivers are
  /  mapped to high-level interface
  /****************************************************************************/
#ifdef INCLUDE_SCI
	#ifdef DSP56832XEVM
	// open SCI1
	DriverFD = OPEN(BSP_LOW_DEVICE_NAME_SCI_1, O_RDWR, sci_getBaudRate(BOOT_SCI_BAUD_RATE));	
	periphBitSet(GPIO_B0 | GPIO_B1, &ArchIO.Sim.GPSReg);
	#else
	// open SCI0			
	DriverFD = OPEN(BSP_LOW_DEVICE_NAME_SCI_0, O_RDWR, sci_getBaudRate(BOOT_SCI_BAUD_RATE));	
	#endif
    bootSendXON();
#else  
	// open Flex Can device driver for Rx and TX
	DriverFD = OPEN(BSP_DEVICE_NAME_CAN_0, O_RDWR, NULL);	
#endif 

#ifdef USE_HYPERTERMINAL
  /****************************************************************************
  / write out banner and version number
  /****************************************************************************/
	if(bootVersionPrompt == true)
    {
	    WRITE(DriverFD, StrCopyright_1, sizeof(StrCopyright_1));
	    WRITE(DriverFD, StrWaitingSrec, sizeof(StrWaitingSrec));
	} 
#endif

   /****************************************************************************
  / Initialize TimerA and set delay value. This value specifies the amount of 
  / time that the bootloader will wait to begin receiving code to be programmed.
  / If no code is received before this timeout, the bootloader will begin execution 
  / of the application already loaded in Program Flash.
  /****************************************************************************/
	if(StartDelay != 0x00FF)
	{
		bootTimerSet(StartDelay);
	}

  /****************************************************************************
  / wait for SCI reception byte or CAN reception message or if time-out 
  / jump to user application
  /****************************************************************************/
	while(!bootReadState())
	{
		bootTimerCheck();
	}

  /****************************************************************************
  / open Flash device driver for Data and Program memory
  /****************************************************************************/
	DataFlashFD = flashXmemOpen(BSP_DEVICE_NAME_FLASH_XMEM, O_READ_COMPARE);
	ProgFlashFD = flashPmemOpen(BSP_DEVICE_NAME_FLASH_PMEM, O_READ_COMPARE);
	
  /****************************************************************************
  / load first line of S-record
  /****************************************************************************/
	bootLoadSRecord(CharBuffer, &DataAddress, &DataLength, DataBuffer);

  /****************************************************************************
  / set data flag if S-record file contains DATA ONLY 
  / S-record file with data only contains S0 line with hex code of 'DATA' word
  /****************************************************************************/
	if(DataLength == 4)
	{
		bootXDATAFlag = true;
	}
	
	bootContinueFlag = true;

  /****************************************************************************
  / erase the PROGRAM FLASH
  /****************************************************************************/
	ioctlFLASH_RAW_MASSERASE_PMEM(ProgFlashFD, NULL);

#ifndef USE_HYPERTERMINAL	
  /****************************************************************************
  / write out banner and version number
  /****************************************************************************/
	if(bootVersionPrompt == true)
    {
	    WRITE(DriverFD, StrCopyright_1, sizeof(StrCopyright_1));
	    WRITE(DriverFD, StrWaitingSrec, sizeof(StrWaitingSrec));
	} 
#endif

  /****************************************************************************
  /  while loop to continue loading S-record
  /****************************************************************************/
	while(bootContinueFlag == true)
	{
  /****************************************************************************
  / load a line of the S-record
  /****************************************************************************/		
		bootLoadSRecord(CharBuffer, &DataAddress, &DataLength, DataBuffer);
	
  /****************************************************************************
  / check to see if the data field is composed of memory loadable data 
  /****************************************************************************/	
		if(CharBuffer[1] == '3')
		{
			if(bootXDATAFlag == true)
			{ 
				DataAddress |= DATA_RAM_ADDRESS_MASK;
    		}                

  /****************************************************************************
  / write date from S-record to memory
  /****************************************************************************/
			bootMemoryWrite(DataAddress, DataBuffer, DataLength);

  /****************************************************************************
  / display the loading progress message 
  /****************************************************************************/
			StrBuffer[0] = BOOT_INDICATION_SYMBOL;
			WRITE(DriverFD, StrBuffer, 1);
		}
  /****************************************************************************
  / there is no data field exit while loop
  /****************************************************************************/	
		else if(CharBuffer[1] == '7')   
		{
			NewStartAddress = DataAddress;
			bootContinueFlag = false;
		}
	}      

  /****************************************************************************
  / display the loading value message and application started message 
  /****************************************************************************/
	StartAddress = NewStartAddress;

#ifdef INCLUDE_SCI 
	bootSendXON();
#endif

    WRITE(DriverFD, StrLoaded_2,  sizeof(StrLoaded_2) - 1);
	
	bootHexToAscii((unsigned char)(DataCounter     >> 16), &StrBuffer[0]);
	bootHexToAscii((unsigned char)(DataCounter     >> 8) , &StrBuffer[2]);
	bootHexToAscii((unsigned char)(DataCounter)          , &StrBuffer[4]);	
	
	StrBuffer[7] = (unsigned char)' ';
	StrBuffer[8] = (unsigned char)' ';	
	
	WRITE(DriverFD, StrBuffer, 9); 
	
	WRITE(DriverFD, StrLoaded_3, sizeof(StrLoaded_3) - 1);
	WRITE(DriverFD, StrStarted_1,  sizeof(StrStarted_1) - 1);

	bootHexToAscii((unsigned char)(NewStartAddress >> 8) , &StrBuffer[0]);	
	bootHexToAscii((unsigned char)(NewStartAddress)      , &StrBuffer[2]);
	StrBuffer[4] = (unsigned char)' ';
	StrBuffer[5] = (unsigned char)' ';
	
	WRITE(DriverFD, StrBuffer, sizeof(StrBuffer));

  /****************************************************************************
  / if there is PROGRAM DATA write user application start address 
  /****************************************************************************/ 	
	if(!bootXDATAFlag)
	{
		ioctlFLASH_CMD_SEEK(DataFlashFD, 2 * (BOOT_START_ADDRESS_PLACE - DATAFLASH_BEGIN_ADDRESS));
		flashWrite(DataFlashFD, &StartAddress, 4);
	}

  /****************************************************************************
  / restore peripheral used to power on reset state and jump to user app 
  /****************************************************************************/ 
	bootExit();
}

/***************************************************************************
* Functions:   bootExit()
*
* Description: switch interrupt vector table 
*              and start user's application

* Returns:     None
*
* Notes:   
*
**************************************************************************/
void bootExit(void)
{
	int i = 0;
	int j = 0;
	int k = 0;

  /****************************************************************************
  / close peripheral used to load S-record file
  /****************************************************************************/ 
	if(DriverFD != IO_INVALID_HANDLE)
	{
		CLOSE(DriverFD); 
	}
	
  /****************************************************************************
  / restore the power on reset state to used peripheral registers    
  /****************************************************************************/ 
 	for(i = 0; i < archoff_registers_num; i++)
	{
		for(j = 0; j < peripheralReg[i].length; j ++)
		{
			periphMemWrite( RegState[k],  (volatile UWord16 *) (peripheralReg[i].StartAddress + j));
			k++;
		}
	} 

  /****************************************************************************
  / PLL is reprogrammed to its initial state after user's application
  / downloading   
  /****************************************************************************/ 
	pllInitialization();

  /****************************************************************************
  / change base address of the interrupt vector table for user's application 
  /****************************************************************************/ 
    periphMemWrite(USER_INTERRUPT_VECTOR_ADDRESS >> 8, &ArchIO.IntCtrl.VectorBaseAddrReg);

  /****************************************************************************
  / call user's application 
  /****************************************************************************/ 
    asm(move.l StartAddress, N);
    asm(jmp (N));
}

/***************************************************************************
* Functions:   bootReadState()
*
* Description: read input queue state
*
* Returns:     1 - queue has any byte
*              0 - queue is empty
*
* Notes:   
*
**************************************************************************/
bool bootReadState()
{
	if(IOCTL(DriverFD, GET_BUFFER_STATUS) != 0)
	{
	    return true;
	}
	else
	{
		return false;
	}
}

/***************************************************************************
* Functions:   bootTimerSet()
*
* Description: Setup and run TIMER A
*
* Returns:
*
* Notes:       timeValue must be defined in sec
*
**************************************************************************/
void bootTimerSet(UWord16 timeValue)
{   
	UWord16 timeDelay;
	   
  /****************************************************************************
  / period of Timer A1 = (128 * 65535)/(2 * PLL_MUL * 1e+6) sec or ~ 8.5/(2 * PLL_MUL)  
  /****************************************************************************/ 
	timeDelay = timeValue * (UWord16)(PLL_MUL * 4 / 17);
    
  /****************************************************************************
  / if timeDelay overflowed then timeDelay = max value  
  /****************************************************************************/ 
	if(timeValue > timeDelay)
	{ 
		timeDelay = 0xFFFF;       
	}
	
	timer_qt_writeLoadReg    (&ArchIO.TimerA.Channel0, 0);         // value to initialize the counter                         
	timer_qt_writeCounterReg (&ArchIO.TimerA.Channel0, 0);         // counter0 set to $0    
	timer_qt_writeCompareReg1(&ArchIO.TimerA.Channel0, 0xFFFF);
	
	timer_qt_writeLoadReg    (&ArchIO.TimerA.Channel1, 0);         // value to initialize the counter                         
	timer_qt_writeCounterReg (&ArchIO.TimerA.Channel1, 0);         // counter1 set to $0    
	timer_qt_writeCompareReg1(&ArchIO.TimerA.Channel1, timeDelay);
		
	timer_qt_start(&ArchIO.TimerA.Channel1,0xE860); 	// enabled  timer A1     
	                                                    // [12:09]  = 0100 Count Source Channel0 output            
                                                        // [8:7]    = 00                                           
														// [6]      = 1 ONCE Count until compare                   
														// [5]      = 1 LENGTH Count until compare then reintialize 
														// [4]      = 0 DIR Count up                                
														// [3]      = 0                                             
														// [2:0]    = 000 OFLAG asserted                          
	
	
	timer_qt_start(&ArchIO.TimerA.Channel0,0x3E00);     // enabled  timer A0      
	                                                    // [12:09]  = 1111 Count Source IPbus clock divide by 128    
														// [8:7]    = 00                                             
														// [6]      = 0 ONCE Count repeatedly                       
														// [5]      = 0 LENGTH Roll over                            
														// [4]      = 0 DIR Count up                                 
														// [3]      = 0                                              
														// [2:0]    = 000 OFLAG asserted                             

}
	
/***************************************************************************
* Functions:   bootTimerCheck()
*
* Description: Check TIMER A
*
* Returns:     1 - timer is reached defined timevalue
*              0 - count in progress
*
* Notes:
*
**************************************************************************/
UWord16 bootTimerCheck(void) 
{   
	if(periphMemRead(&ArchIO.TimerA.Channel1.StatusControlReg) & 0x8000)
	{
		bootExit();

	    return 1;
	}
	else
	{
		return 0;
	}
}

/***************************************************************************
* Functions:   bootAsciiToHex()
*
* Description: transform ASCII code to the character. 
*
* Returns:     Hex code of symbol
*
* Notes:
*
**************************************************************************/
UWord32 bootAsciiToHex(unsigned char * strbuf, char length)
{
	unsigned long value = 0L;
	unsigned char var;

	while(length != 0)
	{
		length--;
		var = *strbuf;
        
		if(var > 0x2F && var < 0x3A)
		{     
			var -= 0x30;                        // ASCII characters '0', '1', ... , '9' 
		}
		else
		{
			var |= 0x20;                        // ASCII characters 'A', 'B', ... , 'F' to 'a', 'b', ... , 'f' 
			if(var > 0x60 && var < 0x67)
			{ 
				var -= 0x57;                    // ASCII characters 'a', 'b', ... , 'f' 
            }
			else
			{
				bootError(INDICATE_ERROR_CHARACTER);
			}
		}
        
		value |= (UWord32)var << (4 * length);
		strbuf++;
	}                        
   
	return value;
}

/***************************************************************************
* Functions:   bootHexToAscii()
*
* Description: transform character to the ASCII code. 
*
* Returns:     None
*
* Notes:
*
**************************************************************************/
void bootHexToAscii(unsigned char value, unsigned char * strbuf)
{
	strbuf[1] = HexTable[ value & 0x0F ];
	strbuf[0] = HexTable[ (value & 0xF0) >> 4 ];
}

/***************************************************************************
* Functions:   bootMemoryWrite()
*
* Description: write data to memory 
*
* Returns:
*
* Notes:
*
**************************************************************************/
void bootMemoryWrite(UWord32 address, UInt8 * buffer, UWord16 length)
{
	UWord16 i;
	UWord16 SizeInWords;
	UWord32 endAddress;
    
    SizeInWords = (length + 1) / 2;
    endAddress  = address + SizeInWords;

	if(address & DATA_RAM_ADDRESS_MASK)
	{
  /****************************************************************************
  / remove mask from address
  /****************************************************************************/  
		address &= ~DATA_RAM_ADDRESS_MASK; 
		endAddress = address + SizeInWords;

  /****************************************************************************
  / write data buffer to DATA FLASH
  /****************************************************************************/  	
		if(address >= DATAFLASH_BEGIN_ADDRESS && endAddress <= (DATAFLASH_END_ADDRESS+1))
		{
			ioctlFLASH_CMD_SEEK(DataFlashFD, (address - DATAFLASH_BEGIN_ADDRESS ) * 2);    
			flashWrite(DataFlashFD, buffer, length);
			DataCounter += length;
            
  /****************************************************************************
  / verify DATA flash memory 
  /****************************************************************************/ 
            ioctlFLASH_CMD_SEEK(DataFlashFD, (address - DATAFLASH_BEGIN_ADDRESS) * 2);
             
  /****************************************************************************
  / flashRead will COMPARE ONLY, because DataFlashFD was opened with 
  / O_READ_COMPARE flag
  /****************************************************************************/ 			
			if(flashRead(DataFlashFD, buffer, length) == -1)
			{
				bootError(INDICATE_ERROR_FLASH);
			}
		}
        else
        { 
            DataInRAM = true;
		} 
	}    
	else
	{
		if(endAddress < (UWord32)&_Pcode_begin_addr_in_ROM || address > (UWord32)&_Pcode_end_addr_in_ROM)
		{
			if(address >= PROGRAMFLASH_BEGIN_ADDRESS && endAddress <= PROGRAMFLASH_END_ADDRESS)
            {
				ioctlFLASH_CMD_SEEK(ProgFlashFD, address * 2);
				flashWrite(ProgFlashFD, buffer, length);
				DataCounter += length;

  /****************************************************************************
  /	verify PROGRAM flash memory 
  /****************************************************************************/ 
				ioctlFLASH_CMD_SEEK( ProgFlashFD, address * 2 );
				
				if(flashPCompare( ProgFlashFD, buffer, length) == -1)
				{
					bootError(INDICATE_ERROR_FLASH);
				}
            }
			else
			{ 
				DataInRAM = true;
			}
		}
	}
}

/*****************************************************************************
*
* Functions:      bootError()
*
* Description:    Indicate number of bootloader error via Serial line.
*                 Perform processor reset.
*
* Returns:        Function does not return control.
*
* Arguments:      ErrorNumber
*
* Notes:
*
*****************************************************************************/
void bootError(UWord16 errorNumber)
{
	char BootErrorMessage[13];
	int  i = 0;
	
	bootHexToAscii(errorNumber, StrBuffer);

	for(i = 0; i < sizeof(StrError_1); i++)
	{
		BootErrorMessage[i] = StrError_1[i];
	}
	
	BootErrorMessage[11] = StrBuffer[0];
	BootErrorMessage[12] = StrBuffer[1];

	WRITE(DriverFD, BootErrorMessage, sizeof(BootErrorMessage));

	while(1)
	{
	}
}

/***************************************************************************
 * Functions:   bootLoadSRecord()
 *
 * Description: loads one S-Record line
 *              checks timeout if required
 *
 * Returns:     None
 *
 * Notes:       None
 *
 **************************************************************************/
void bootLoadSRecord(UInt8* srecType, UWord32* dataAddress, UWord16* dataLength, UInt8* dataBuffer)
{
	bool    bootContinueFlag = true;
	UInt8   DataChecksum;
	UInt8   DataChecksumReal;
	UWord16 i;
	UInt8   CharBuffer[10];

	srecType[0] = 0;

#ifdef INCLUDE_SCI 
	bootSendXON();
#endif 
	
	while(bootContinueFlag == true) 
	{
		bootBlockRead(srecType, 1);

        if((srecType[0] | 0x20) == 's')
		{
			bootBlockRead(&srecType[1], 1);
			
			if((srecType[1] == '3') || (srecType[1] == '0') || (srecType[1] == '7'))
			{
				SRecordChecksum = 0;

  /****************************************************************************
  /	read from input device and converts into bytes
  /****************************************************************************/ 
				bootBlockReadWithConvert(CharBuffer, 1, false);
				
  /****************************************************************************
  /	S-record length minus address and checksum fields
  /****************************************************************************/ 
				*dataLength = (UInt8)(CharBuffer[0]) - 5;          

				bootBlockReadWithConvert((UInt8*)((( UWord32 )dataAddress) * 2), 4, true);
			
  /****************************************************************************
  /	receive data word, LSB first
  /****************************************************************************/ 		 
				bootBlockReadWithConvert(dataBuffer, *dataLength, false);  
				
				DataChecksum = SRecordChecksum;

  /****************************************************************************
  /	read from input device and converts into bytes
  /****************************************************************************/ 
				bootBlockReadWithConvert(&DataChecksumReal, 1, false);

#ifdef INCLUDE_SCI 
				bootSendXOFF();
#endif 
				DataChecksum = 0xFF - (DataChecksum & 0xFF);

				if(DataChecksumReal != DataChecksum)
				{
				    bootError(INDICATE_ERROR_CHECKSUM);
				}
				
				bootContinueFlag = false;
			}
			else
			{
				bootError(INDICATE_ERROR_FORMAT);
			}
		}
		else if((srecType[0] != 0x0a) && (srecType[0] != 0x0d))
		{
			bootError(INDICATE_ERROR_CHARACTER);
		}
	}
}

/***************************************************************************
 * Functions:   bootBlockRead()
 *
 * Description: read block from input device
 *
 * Returns:
 *
 * Notes:
 *
 **************************************************************************/
UWord16 bootBlockRead(UInt8 * pBuffer, UWord16 Size)
{
    UWord16 ReturnValue = Size;

  	ReturnValue =READ(DriverFD, pBuffer, Size);  

	return ReturnValue;
}

/***************************************************************************
* Functions:   bootBlockReadWithConvert()
*
* Description: read block from input device and converts into bytes
*
* Returns:
*
* Notes:
*
**************************************************************************/
UWord16 bootBlockReadWithConvert(UInt8 * buffer, UWord16 length, bool isReverse)
{
	UWord16 BlockLength = length;
	UInt8   TmpBuff[2];
    
	if(isReverse == true)
	{
		buffer = buffer + (length - 1);
	}
    
	while(length != 0)
	{
		bootBlockRead(TmpBuff, 2);
		
		*buffer = bootAsciiToHex(TmpBuff, 2);
		
		SRecordChecksum = SRecordChecksum + *buffer;
		
		if(isReverse == true)
		{
			buffer--;
		}
		else
		{
			buffer++;
		}
		length--;
	}

	return BlockLength;
}

/***************************************************************************
* Functions:   pllInitialization()
*
* Description: initializes PLL
* Returns:
*
* Notes:
*
***************************************************************************/
void pllInitialization(void)
{
  /****************************************************************************
  / set lock detector on and choose core clock 
  /****************************************************************************/ 
	asm(move.w	#PLL_LOCK_DETECTOR | PLL_ZCLOCK_PRESCALER, X:(BSP_PERIPH_BASE + archoff_Pll_ControlReg));
	
  /****************************************************************************
  / if LORTP is zero we have to save old value (prevent generate of lock
  / interrupted) 
  /****************************************************************************/ 
	asm(bfclr #0x0FFF, X:(BSP_PERIPH_BASE + archoff_Pll_DivideReg));
	asm(bfset #PLL_DIVIDE_BY_REG_INI, X:(BSP_PERIPH_BASE + archoff_Pll_DivideReg));
	
  /****************************************************************************
  / set x0 with timeout value  
  /****************************************************************************/ 
	asm(move.w  #0x4000, X0);            

	asm(wait_pll:);
	asm(dec.w X0);
	asm(brset #PLL_STATUS_LOCK_0, X:(BSP_PERIPH_BASE + archoff_Pll_StatusReg), end_wait);
	asm(cmp.w #0, X0);
	asm(bne wait_pll);

  /****************************************************************************
  / PLL did not lock, but proceed anyway 
  /****************************************************************************/  
	asm(end_wait:);
	asm(move.w	#PLL_CONTROL_REG_INI, X:(BSP_PERIPH_BASE + archoff_Pll_ControlReg));
}
