#if defined(DSP56838EVM) 
/*******************************************************************************
*
* Motorola Inc.
* (c) Copyright 2002 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
********************************************************************************
*
* FILE NAME: mscan.c
*
*******************************************************************************/
#include "port.h"
#include "arch.h"
#include "periph.h"
#include "io.h"

#include "stdio.h"
#include "stdlib.h"
#include "stdarg.h"
#include "string.h"

#include "bsp.h"

#include "supconfig.h"
#include "config.h"

#include "mscan.h"

/* MSCAN property resolved table: 1 - free, 0 - busy */
const UWord16 CANDll [/* 8 */] = {
/* 000 */  0x00 | CANM_TXISR | CANM_ALLBUSY,
/* 001 */  0x01 | CANM_TXISR,           
/* 010 */  0x02 | CANM_TXISR | CANM_FREE1,
/* 011 */  0x01,            
/* 100 */  0x04 | CANM_TXISR | CANM_FREE2,
/* 101 */  0x01,            
/* 110 */  0x02 |             CANM_FREE1,
/* 111 */  0x01 |             CANM_ALLFREE /* all free - reset priority */
};

extern const io_sMSCANInterface canInterfaceVT; 
/* extern const io_sDriver interfaceCAN; */

static UWord16 CANInit();
static UWord16 CANEvlFlag( UWord16 flg );
static UWord16 CANSetFilter();
static can_sReadBuffer* CANGetFreeReadBuffer( );
static can_sWriteBuffer* CANGetFreeWriteBuffer( );

static int CANFilterWeight( CANADDRESSTYPE canID, CANADDRESSTYPE code, CANADDRESSTYPE mask );
static void CANTransmit( UWord16 idx, can_sWriteBuffer* pBuf, can_sData* pData );
static can_sWriteBuffer* TakeAway( can_sWriteBuffer* pBuf );
static int CheckCANID( CANADDRESSTYPE canID, UWord16 mode );
static CANADDRESSTYPE GetCANID( volatile UWord16* pAddr );
static void SetCANID( volatile UWord16* pAddr, CANADDRESSTYPE canID );

extern const can_sInitialState canInitialState;   	

/* the number of used buffers is defined statically */
/* to control of open devices */
static can_sWriteBuffer* pLastWrite;
static can_sReadBuffer*  pLastRead;

/* CAN filters in CAN ID format */
static struct CANFilterMirror  sCANFilterMirror[CANFILTERLIMIT];

extern can_pRawCallBack CANRcvPrepare;  /*  RAW CallBack pointer */

/* CAN driver context variables */
int CANErrno;                           /* last error code */
static UWord16 actFilterLimit ;         /* free/busy filter index */
static can_sInitialState sIState;       /* MSCAN initial state */
static UWord16 msprio;                  /* priority support counter */

/*******************************************************************************
*
* NAME: CANStdRcvPrepare
*
* DESCRIPTION: To find driver receive buffer by incoming CANID
*
********************************************************************************
* PARAMETERS:   incoming CAN ID 
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: Can be implemented by asm
*
* DEPENDENCIES: None
*******************************************************************************/
can_sData* CANStdRcvPrepare( CANADDRESSTYPE canID )
{

	can_sReadBuffer* pBuf;
	register can_sData* pData = 0;
	
    for( pBuf = sIState.pRxBuffer;
                pBuf < pLastRead;
                        pBuf = (can_sReadBuffer*)((char*)pBuf + sIState.RxSize))
    {
        if( pBuf->canID == canID )
        {
            pData = &pBuf->data[0];
            if( pData->length != 0 )
                pBuf->flags |= CAN_BUF_OVERFLOW; 
            else
                pBuf->flags |= CAN_BUF_FULL; 
        }
    }
    return pData;
   
}

/*******************************************************************************
*
* NAME: CANEvlFlag
*
* DESCRIPTION: Evaluate CAN flags to obtain interrupt flags
*
********************************************************************************
* PARAMETERS:   current flag set 
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: 
*
* DEPENDENCIES: None
*******************************************************************************/
UWord16 CANEvlFlag( UWord16 flg )
{
    return(  (CANWUPIE | CANRXFIE) | ~( flg | ~CANWATCHFLAG )   );
}

/*******************************************************************************
*
* NAME: GetCANID
*
* DESCRIPTION: Compose 11/29-bit CAN ID from the registers values
*
********************************************************************************
* PARAMETERS:   Address of the registers 
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: 
*
* DEPENDENCIES: None
*******************************************************************************/
CANADDRESSTYPE GetCANID( volatile UWord16* pAddr )
{
#if defined( CAN20B )
    /*  29-bit CAN ID is taken from four MSCAN registers in the following way:
        xxxxxxxx                                IdReg0
                 xxx11xxx                       IdReg1
                          xxxxxxxx              IdReg2
                                    xxxxxxx0    IdReg3
        xxxxxxxx  xxxxxx  xxxxxxxx  xxxxxxx     29-bit CAN ID  */

        return        ( (UWord32)periphMemRead(&pAddr[0]) << 21)
                    | (((UWord32)periphMemRead(&pAddr[1]) & 0xE0) << 13)
                    | (((UWord32)periphMemRead(&pAddr[1]) & 0x7) << 15)
                    | ( (UWord32)periphMemRead(&pAddr[2]) << 7)
                    | ( (UWord32)periphMemRead(&pAddr[3]) >> 1)
                    ;
#else /*  defined (CAN20B) */
    return ( periphMemRead(&pAddr[0]) << 3 )
                    | ( periphMemRead(&pAddr[1]) >> 5 );
#endif /* defined (CAN20B) */
}

/*******************************************************************************
*
* NAME: SetCANID
*
* DESCRIPTION: Decompose 11/29-bit CAN ID into the registers 
*
********************************************************************************
* PARAMETERS:   Address of the registers, CAN ID
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: 
*
* DEPENDENCIES: None
*******************************************************************************/
void SetCANID( volatile UWord16* pAddr, CANADDRESSTYPE canID )
{
#if defined( CAN20B )
	/* if (canID>0x07FF) */
    /*  29-bit CAN ID is placed into four MSCAN registers in the following way:
        xxxxxxxx                                IdReg0
                 xxx11xxx                       IdReg1
                          xxxxxxxx              IdReg2
                                    xxxxxxx0    IdReg3
        xxxxxxxx  xxxxxx  xxxxxxxx  xxxxxxx     29-bit CAN ID  */
        
    periphMemWrite( (UWord16)(canID >> 21), &pAddr[0] );

    /* check carefully ! */

    periphMemWrite( 
        (UWord16)(( ((canID >> 15) & 0x7) | 0x18 | ((canID >> 13) & 0xE0))), &pAddr[1] );
    periphMemWrite( (UWord16)(canID >> 7), &pAddr[2] );
    periphMemWrite( (UWord16)(canID << 1), &pAddr[3] );

#else /*  defined (CAN20B) */

    periphMemWrite( canID >> 3, &pAddr[0]);   
    periphMemWrite( canID << 5, &pAddr[1]); 

#endif /* defined (CAN20B) */
}

/*******************************************************************************
*
* NAME: CANSetFilter
*
* DESCRIPTION: Set CANCIDARs and CANCIDMRs
*
********************************************************************************
* PARAMETERS:   
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: 
*
* DEPENDENCIES: None
*******************************************************************************/
UWord16 CANSetFilter() 
{
#if defined(CAN20B)
        SetCANID( &ArchIO.CAN.IdAcceptReg0, sCANFilterMirror[0].code );
        SetCANID( &ArchIO.CAN.IdAcceptReg4, sCANFilterMirror[1].code );

        SetCANID( &ArchIO.CAN.IdMaskReg0, sCANFilterMirror[0].mask );
        SetCANID( &ArchIO.CAN.IdMaskReg4, sCANFilterMirror[1].mask );

		if( actFilterLimit > 0)
			periphBitSet(0x1, &ArchIO.CAN.IdMaskReg3 );
		if( actFilterLimit > 1)
			periphBitSet(0x1, &ArchIO.CAN.IdMaskReg7);

#else /* defined (CAN20B) */

        SetCANID( &ArchIO.CAN.IdAcceptReg0, sCANFilterMirror[0].code );
        SetCANID( &ArchIO.CAN.IdAcceptReg2, sCANFilterMirror[1].code );
        SetCANID( &ArchIO.CAN.IdAcceptReg4, sCANFilterMirror[2].code );
        SetCANID( &ArchIO.CAN.IdAcceptReg6, sCANFilterMirror[3].code );

        SetCANID( &ArchIO.CAN.IdMaskReg0, sCANFilterMirror[0].mask );
        SetCANID( &ArchIO.CAN.IdMaskReg2, sCANFilterMirror[1].mask );
        SetCANID( &ArchIO.CAN.IdMaskReg4, sCANFilterMirror[2].mask );
        SetCANID( &ArchIO.CAN.IdMaskReg6, sCANFilterMirror[3].mask );

		if( actFilterLimit > 0)
			periphBitSet(0x7, &ArchIO.CAN.IdMaskReg1 );
		if( actFilterLimit > 1)
			periphBitSet(0x7, &ArchIO.CAN.IdMaskReg3 );
		if( actFilterLimit > 2) 
			periphBitSet(0x7, &ArchIO.CAN.IdMaskReg5 );
		if( actFilterLimit > 3)
			periphBitSet(0x7, &ArchIO.CAN.IdMaskReg7);

#endif /* defined (CAN20B) */

return 0;
}

/*******************************************************************************
*
* NAME: CANInit
*
* DESCRIPTION: Initialize MSCAN device
*              
********************************************************************************
* PARAMETERS:   
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: 
*
* DEPENDENCIES: None
*******************************************************************************/
static UWord16 CANInit()
{
    UWord16 i;
    can_sWriteBuffer* pWriteBuf;
    can_sReadBuffer* pReadBuf;

    periphBitSet(CANSFTRES, &ArchIO.CAN.Control0Reg);         /* set soft reset bit */ 

    for( pWriteBuf = sIState.pTxBuffer; 
                pWriteBuf < pLastWrite; 
                        pWriteBuf++ )
    {
        if( pWriteBuf->flags & ( CAN_IDX_BUF0 | CAN_IDX_BUF1 | CAN_IDX_BUF2 | CAN_BUF_PENDING )  )
        {
            pWriteBuf->flags |= CAN_BUF_ERROR;
            pWriteBuf->flags &= ~( CAN_IDX_BUF0 | CAN_IDX_BUF1 | CAN_IDX_BUF2 | CAN_BUF_PENDING );
        }
    }

    for( pReadBuf = sIState.pRxBuffer;
                pReadBuf < pLastRead;
                        pReadBuf++ )
    {
        pReadBuf->data[0].length = 0;
        pReadBuf->flags &= ~( CAN_BUF_FULL | CAN_BUF_OVERFLOW );
    }


    msprio = CANLOWESTPRIO;

    periphMemWrite( sIState.regCMCR1,  &ArchIO.CAN.Control1Reg ); 
    periphMemWrite( sIState.regCBTR0,  &ArchIO.CAN.BusTiming0Reg ); 
    periphMemWrite( sIState.regCBTR1,  &ArchIO.CAN.BusTiming1Reg ); 
    periphMemWrite( CANFILTERMODE,     &ArchIO.CAN.IdControlReg ); 

    CANSetFilter();
    
    periphBitClear(CANSFTRES, &ArchIO.CAN.Control0Reg ); /* clear SFTRES bit */
    periphMemWrite( sIState.regCMCR0 & 0x28 , &ArchIO.CAN.Control0Reg );
    
    periphMemWrite( 0,  &ArchIO.CAN.TxControlReg );  
    
    /* no Tx interrupt, controling by  handler */
    
    periphMemWrite( CANWATCHFLAG, &ArchIO.CAN.RxFlagReg ); 
    periphMemWrite( CANEvlFlag( periphMemRead(&ArchIO.CAN.RxFlagReg) ), 
                                                &ArchIO.CAN.RxIntEnableReg );
    
    return 0;
}


/*******************************************************************************
*
* NAME: CANReceiveISR
*
* DESCRIPTION: receive ISR handler
*
********************************************************************************
* PARAMETERS:   
*
* SIDE EFFECTS: Slowest handler in the driver, impacts to the interrupt lattency
*
* DESIGNER NOTES: copy is not effective on C, recomended to use asm
*
* DEPENDENCIES: None
*******************************************************************************/
void CANReceiveISR(void)
{
    register  can_sData* pCrd;

    while( periphBitTest(CANRXF, &ArchIO.CAN.RxFlagReg) )     /* RX buffer ready */
    {
      pCrd = (*CANRcvPrepare)( GetCANID(&ArchIO.CAN.RxBuffer.IdReg0) ); /* CANStdRcvPrepare */
        if( pCrd ) 
        {   
            pCrd->length =  ( periphMemRead(&ArchIO.CAN.RxBuffer.DataLengthReg) ) & 0x000F;
            pCrd->data[0] = periphMemRead(&ArchIO.CAN.RxBuffer.DataSegReg[0]);
            pCrd->data[1] = periphMemRead(&ArchIO.CAN.RxBuffer.DataSegReg[1]);
            pCrd->data[2] = periphMemRead(&ArchIO.CAN.RxBuffer.DataSegReg[2]);
            pCrd->data[3] = periphMemRead(&ArchIO.CAN.RxBuffer.DataSegReg[3]);
            pCrd->data[4] = periphMemRead(&ArchIO.CAN.RxBuffer.DataSegReg[4]);
            pCrd->data[5] = periphMemRead(&ArchIO.CAN.RxBuffer.DataSegReg[5]);
            pCrd->data[6] = periphMemRead(&ArchIO.CAN.RxBuffer.DataSegReg[6]);
            pCrd->data[7] = periphMemRead(&ArchIO.CAN.RxBuffer.DataSegReg[7]);
        }
        periphBitSet( CANRXF, &ArchIO.CAN.RxFlagReg );   /* free RX */
    }
    return;
}


/*******************************************************************************
*
* NAME: CANSendISR
*
* DESCRIPTION: send ISR handler
*
********************************************************************************
* PARAMETERS:   
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: 
*
* DEPENDENCIES: None
*******************************************************************************/
void CANSendISR(void)
{
    register UWord16 idx;
    can_sWriteBuffer* pBuf;

    idx = periphMemRead(&ArchIO.CAN.TxControlReg) & periphMemRead(&ArchIO.CAN.TxFlagReg);

    idx = CANDll[ idx & 0x07 ] & 0x07;

    periphBitClear(idx, &ArchIO.CAN.TxControlReg ); /* clear corresponded flag */

    periphMemWrite( CANWATCHFLAG, &ArchIO.CAN.RxFlagReg ); /* clear error if possible */
    periphMemWrite( CANEvlFlag( periphMemRead(&ArchIO.CAN.RxFlagReg) ),
                                             &ArchIO.CAN.RxIntEnableReg );
}

/*******************************************************************************
*
* NAME: CANTransmit
*
* DESCRIPTION: CAN send buffer 
*
********************************************************************************
* PARAMETERS:   
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: 
*
* DEPENDENCIES: None
*******************************************************************************/
void CANTransmit( UWord16 idx, can_sWriteBuffer* pBuf, can_sData* pData )
{
    arch_sMSCANBuffer* pHBuf = &ArchIO.CAN.TxBuffer[ idx >> 1 ]; /* 1, 2, 4 -> 0, 1, 2 */
    pBuf->flags &= ~( CAN_IDX_BUF0 | CAN_IDX_BUF1 | CAN_IDX_BUF2 | CAN_BUF_ERROR | CAN_BUF_PENDING );
    pBuf->flags |= idx;
    SetCANID( &pHBuf->IdReg0, pBuf->canID );

    periphMemWrite( pData->length ,  &pHBuf->DataLengthReg );
    periphMemWrite( pData->data[0],  &pHBuf->DataSegReg[0] );
    periphMemWrite( pData->data[1],  &pHBuf->DataSegReg[1] );
    periphMemWrite( pData->data[2],  &pHBuf->DataSegReg[2] );
    periphMemWrite( pData->data[3],  &pHBuf->DataSegReg[3] );
    periphMemWrite( pData->data[4],  &pHBuf->DataSegReg[4] );
    periphMemWrite( pData->data[5],  &pHBuf->DataSegReg[5] );
    periphMemWrite( pData->data[6],  &pHBuf->DataSegReg[6] );
    periphMemWrite( pData->data[7],  &pHBuf->DataSegReg[7] );
    periphMemWrite( pBuf->priority,  &pHBuf->U.TxBufferPriorityReg );

        /* hardware send */
    periphMemWrite(idx, &ArchIO.CAN.TxFlagReg);    /* send */
}



/*******************************************************************************
*
* NAME: CANErrorISR
*
* DESCRIPTION: error ISR handler 
*
********************************************************************************
* PARAMETERS:   
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: 
*
* DEPENDENCIES: None
*******************************************************************************/
void CANErrorISR(void)
{
    if( periphBitTest(CANBOFFIF, &ArchIO.CAN.RxFlagReg) )  /* bus-off ? */
    {
        CANInit();
    }
            /* try to clear all errors */
    periphMemWrite( CANWATCHFLAG, &ArchIO.CAN.RxFlagReg ); 
    periphMemWrite( CANEvlFlag( periphMemRead(&ArchIO.CAN.RxFlagReg) ),
                                             &ArchIO.CAN.RxIntEnableReg );
}



/*******************************************************************************
*
* NAME: CANWakeUpISR
*
* DESCRIPTION: wakeup ISR handler 
*
********************************************************************************
* PARAMETERS:   
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: 
*
* DEPENDENCIES: None
*******************************************************************************/
void CANWakeUpISR(void)  /* CAN Wake up */
{
    periphMemWrite( CANWUPIF, &ArchIO.CAN.RxFlagReg ); /* reset by 1 */
}


/*******************************************************************************
*
* NAME: CANFilterWeight
*
* DESCRIPTION: weight function to define optimal appropriate filter for 
*              incoming CAN ID
********************************************************************************
* PARAMETERS:   CAN ID, current code and mask
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: 
*
* DEPENDENCIES: None
*******************************************************************************/
int CANFilterWeight( CANADDRESSTYPE canID, CANADDRESSTYPE code, CANADDRESSTYPE mask )
{
    CANADDRESSTYPE tmp;
    CANADDRESSTYPE d;
    int i = 0;

    if( code == 0 && mask == 0 )
        return 0; /* not used yet */
    tmp = ( code ^ canID ) & ~mask;  /* distance for the input filter */

    while(1)
    {
        d = ~( ~tmp + 1);
        if( (tmp & d) != 0 )
        {
            tmp &= d;
            i++;
        }
        else return i;
    } 
}

/*******************************************************************************
*
* NAME: CheckCANID
*
* DESCRIPTION: check CAN ID to protect from reopen
*              
********************************************************************************
* PARAMETERS:   CAN ID, mode for loopback
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: 
*
* DEPENDENCIES: None
*******************************************************************************/
int CheckCANID( CANADDRESSTYPE canID, UWord16 mode )
{
    register void* pBuf;

    if( !( mode & (CANLOOPB) >> 1 )   )    
        for( pBuf = sIState.pRxBuffer; 
                pBuf < pLastRead; 
                        ((char*)pBuf) += sIState.RxSize )
    {
        if( ((can_sReadBuffer*)pBuf)->canID == canID)  /* checking receive buffers */
        {
            CANErrno = CAN_ERR_REOPEN;
            return 0;  
        }
    }
    
    if( !( mode & (CANLOOPB)  )   )    
        for( pBuf = sIState.pTxBuffer; 
                pBuf < pLastWrite; 
                        ((char*)pBuf) += sizeof(can_sWriteBuffer)  )
    {
        if( ((can_sWriteBuffer*)pBuf)->canID == canID) /* checking send buffers */
        {
            CANErrno = CAN_ERR_REOPEN;
            return 0;  /* error: re-open is not possible */
        }
    }

    return 1;

}

/*******************************************************************************
*
* NAME: Open
*
* DESCRIPTION: driver Open procedure: open CAN device,  receive and send CAN ID
*              
********************************************************************************
* PARAMETERS:   device name, open flags, pointer to initialization structure
*               of type can_sInitialState
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: following I/O flags accepted: 
*                   O_RDONLY - Message buffer is opened for READ, 
*                   O_WRONLY - Message buffer is opened for WRITE, 
*                               transmission mode is synchronous, 
*                   O_WRONLY | O_NON_BLOCK - Message buffer is opened for WRITE,
*                               transmission mode is asynchronous
*					O_CAN_PRIORITY_SCHEDULE
*
* DEPENDENCIES: None
*******************************************************************************/
handle_t  canOpen(const char * pName, int oFlags, ...)
{
	int i;
	va_list    Args;
	CANADDRESSTYPE canID;

	Args = (char *)&oFlags;

	canID = va_arg(Args, CANADDRESSTYPE);

	va_end(Args);

    if( pName != BSP_DEVICE_NAME_CAN_0 )
    {
        return (handle_t ) -1;
    }

    /***    open for read ( reception )   ***/  
    if( (UWord16)oFlags & O_RDONLY   )
    {
	can_sReadBuffer* pRead;	
	long xi;	
	long xweight = 0xFF;
	long weight;	 
	
              
        if( !CheckCANID( canID, sIState.regCMCR1 & CANLOOPB )   ) 
        {
             CANErrno = CAN_ERR_REOPEN;
             return (handle_t )-1; /* error: no more driver buffers */
        }

        if( pLastRead >= sIState.pLastRx )
        {
            CANErrno = CAN_ERR_NO_BUFFERS;
            return (handle_t )-1; /* error: no more driver buffers */
        }

        pRead = pLastRead;
		
		pLastRead =	(can_sReadBuffer*)((char*) pLastRead + sIState.RxSize);

        pRead->flags = CAN_BUF_READ_HANDLE;

        pRead->drvInterface = (io_sInterface *)&canInterfaceVT; /* interfaceCAN_Rx; */

        pRead->canID = canID; 

        
        if( !((UWord16)oFlags & O_NONBLOCK) )
            pRead->flags |= CAN_BUF_SYNCHRONOUS;
        
        if( actFilterLimit < CANFILTERLIMIT )/* CAN20B can be defines from canID*/
        {
            sCANFilterMirror[actFilterLimit].code = canID;
            sCANFilterMirror[actFilterLimit].mask = 0;
            actFilterLimit++;
        }
        else
        {   /* filter limit is exhausted, looking for appropriate filter */
			
            for( i = 0; i < CANFILTERLIMIT; i++  )
            {
                weight = CANFilterWeight( canID, 
                                     sCANFilterMirror[i].code,
                                     sCANFilterMirror[i].mask
                                    );

                if( weight < xweight )  /* definition of optimal filter */
                {
                    xweight = weight;
                    xi = i;
                }
            }
            sCANFilterMirror[xi].mask |=  
                canID ^ sCANFilterMirror[xi].code;
        }

        periphBitSet( CANSFTRES, &ArchIO.CAN.Control0Reg );       /* set soft reset bit */ 

        /* No need to disable interrupts  since all MSCAN interrupts
           are disabled now 
        */
        periphMemWrite( sIState.regCBTR0, &ArchIO.CAN.BusTiming0Reg );
        periphMemWrite( sIState.regCBTR1, &ArchIO.CAN.BusTiming1Reg );
        periphMemWrite( CANFILTERMODE,    &ArchIO.CAN.IdControlReg );

        CANSetFilter();


        periphBitClear( CANSFTRES, &ArchIO.CAN.Control0Reg ); /* clear SFTRES bit, defined in common mask */
        /* *** */
        periphMemWrite( 0, &ArchIO.CAN.TxControlReg );        /*  no Tx interrupt, controling by DLL handler */
        periphMemWrite( CANWATCHFLAG, &ArchIO.CAN.RxFlagReg );
        periphMemWrite( CANEvlFlag( periphMemRead(&ArchIO.CAN.RxFlagReg ) ),
                                         &ArchIO.CAN.RxIntEnableReg );
                                          
        return (handle_t )pRead; 
    }
    /***   open for write ( transmission )  ***/    
    else if( (UWord16)oFlags & O_WRONLY )
    {
    	UWord16 test = sizeof(can_sWriteBuffer);
        can_sWriteBuffer* pWrite;

        if( !CheckCANID( canID, (sIState.regCMCR1 & CANLOOPB) >> 1 )   ) 
        {
            CANErrno = CAN_ERR_REOPEN;
            return (handle_t )-1; /* error: ID has been opened yet */
        }

        if( pLastWrite >= sIState.pLastTx )
        {
            CANErrno = CAN_ERR_NO_BUFFERS;
            return (handle_t )-1; /* error: no more driver buffers */
        }
        
        
        pWrite = pLastWrite;
        pLastWrite++;  /* next buffer */

        pWrite->drvInterface = (io_sInterface *)&canInterfaceVT; /* interfaceCAN_Tx; */

        pWrite->flags = 0; /* CAN_BUF_WRITE_HANDLE */

        pWrite->canID = canID; 

        if( !((UWord16)oFlags & O_NONBLOCK) )
            pWrite->flags |= CAN_BUF_SYNCHRONOUS;

        /* message schedule block */
        if ((UWord16)oFlags & O_CAN_PRIORITY_SCHEDULE)
        {
            pWrite->flags |= CAN_BUF_PRIORITY_SCHEDULE;

            /* work for restricted ranges */
            pWrite->priority = 
                (UWord16)((pWrite->canID) >> (CANIDSIZE - 7));          
        }

        return (handle_t )pWrite; 
    }

    CANErrno = CAN_ERR_PARAMETER;
    return (handle_t )-1; /* flag error */
    
}

/*******************************************************************************
*
* NAME: Read
*
* DESCRIPTION: Driver read procedure
*              
********************************************************************************
* PARAMETERS:   file descriptor, buffer and size
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: handler is pointer to the bufffer, 
*                 truncation is possible when porting
*                 In case when CAN_8BIT data type is specified, data from one 
*                 receiving register is considered to be of a single 8-bit 
*                 char value.
*                 In case of CAN_16BIT data type the data from two receiving 
*                 registers is composed to a single 16-bit value.
*               
*               
* DEPENDENCIES: None
*******************************************************************************/
ssize_t canRead(handle_t FileDesc,void *pBuffer, size_t size)
{
	//can_sReadBuffer * pHandle = (can_sReadBuffer *)FileDesc;
    can_sReadBuffer* pBuf = (can_sReadBuffer*) FileDesc;
    register char* pSrc;
    
    ssize_t len; 
    
    /* disable source isr */
    periphBitClear( CANRXFIE, &ArchIO.CAN.RxIntEnableReg );

    pSrc = (char *)(&pBuf->data[0]);
        pBuf->flags &= ~(CAN_BUF_FULL | CAN_BUF_OVERFLOW);
    
    len = ((can_sData*)pSrc)->length;
    ((can_sData*)pSrc)->length = 0;  /* mark data as free */

    pSrc = (char *)(((can_sData*)pSrc)->data);

    if( len > size ) 
            len = size;  

    memcpy(pBuffer, pSrc, len );

    /* enable source isr */
    periphBitSet( CANRXFIE, &ArchIO.CAN.RxIntEnableReg );

    return len;
    
}


/*******************************************************************************
*
* NAME: Write
*
* DESCRIPTION: Driver write procedure
*              
********************************************************************************
* PARAMETERS:   file descriptor, buffer and size
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: handler is pointer to the buffer, 
*                 truncation is possible when porting
*
*                 In case when CAN_8BIT data type is specified, data taken from
*                 the buffer is considered to be a 8-bit char, and is placed into 
*                 single transmission register 
*                 In case of CAN_16BIT data type the data from the buffer 
*                 is considered to be of a 16-bit value and is decomposed 
*                 to be placed in two transmission registers.
*               
* DEPENDENCIES: None 
*******************************************************************************/
ssize_t canWrite(handle_t FileDesc, const void *pBuffer, size_t size)
{
	register UWord16 msProp;  /* current property *//* static because board memory or compiller bug */
	static UWord16 save; 	  /* static because board memory or compiller bug */
    can_sWriteBuffer* pBuf = (can_sWriteBuffer*)FileDesc;    
    
    unsigned char *data = (unsigned char*)pBuffer; 
    arch_sMSCANBuffer* pHBuf;
   	UWord16 i;
    
// 	static can_sData       cdata;        /* CAN send data */

    /* wait in case synchronouse */
    if (pBuf->flags & CAN_BUF_SYNCHRONOUS)
    {
	    msProp = 0xFF00;    
   		while( !periphBitTest(CANSYNCH, &ArchIO.CAN.Control0Reg) ) /* synchronization test */
	    {
   		    msProp += 1; /* 1 * 256 delays cycle */
       		if( msProp == 0 ) 
	        {
   		        CANErrno = CAN_ERR_SYNCH;
       		    return -1;
	        }
   		}
	    
	    if( periphBitTest( CANBOFFIF, &ArchIO.CAN.RxFlagReg) ) /* bus-off test */ 
    	{
        	CANErrno = CAN_ERR_BUSOFF;
	        return -1;
    	}
   	}

    msProp = CANDll[ periphMemRead(&ArchIO.CAN.TxFlagReg) & (CANTXE0 | CANTXE1 | CANTXE2)]; /* Check free buffers */

    if( msProp & CANM_ALLFREE ) 
        msprio = CANLOWESTPRIO;    /* if all buffers are free - reset the priority */

    if( !(pBuf->flags & CAN_BUF_PRIORITY_SCHEDULE ) )
    {
        if( msprio >= 256 )
        {
            CANErrno = CAN_ERR_BUSY;
            return( -1 );
        }
        pBuf->priority = msprio;
        msprio++;
    }
    
    if( msProp & CANM_ALLBUSY )
    {               
    	/* all Tx buffers busy */
        CANErrno = CAN_ERR_BUSY;
        return -1;
    }
    
 /* disable interrupts ( ISRs  and errors ) */
    save = periphMemRead( &ArchIO.CAN.TxControlReg);  
    periphMemWrite( 0, &ArchIO.CAN.TxControlReg);  
    periphBitClear( CANERRORFLAG, &ArchIO.CAN.RxIntEnableReg );

    if ( size > 8 ) 
    		size = 8;

//    memcpy( cdata.data, pBuffer, size );
//    cdata.length = size;

    pBuf->flags |= CAN_BUF_PENDING;

    msProp &= 0x07;

/*    CANTransmit( msProp , pBuf, &cdata ); */
	
    pHBuf = &ArchIO.CAN.TxBuffer[ (msProp >> 1) ]; /* 1, 2, 4 -> 0, 1, 2 */  	

  	pBuf->flags &= ~( CAN_IDX_BUF0 | CAN_IDX_BUF1 | CAN_IDX_BUF2 | CAN_BUF_ERROR | CAN_BUF_PENDING );
    pBuf->flags |= msProp;
   	SetCANID( &pHBuf->IdReg0, pBuf->canID );

    periphMemWrite( size ,  &pHBuf->DataLengthReg );

    for(i=0;i<size;i++)
   		periphMemWrite( *data++,  &pHBuf->DataSegReg[i] );
    		
   	periphMemWrite( pBuf->priority,  &pHBuf->U.TxBufferPriorityReg );

    /* hardware send */
    periphMemWrite(msProp, &ArchIO.CAN.TxFlagReg);    /* send */
                        
    /* enable interrupts */
    periphMemWrite( CANWATCHFLAG, &ArchIO.CAN.RxFlagReg ); 
    periphBitSet( msProp | save, &ArchIO.CAN.TxControlReg);   /* isr for identified buffer */
    periphMemWrite( CANEvlFlag( periphMemRead(&ArchIO.CAN.RxFlagReg) ),
                                             &ArchIO.CAN.RxIntEnableReg );
                                                                                          
    /* wait in case synchronouse */
    if (pBuf->flags & CAN_BUF_SYNCHRONOUS)
    {
        while( 1 )
        {
            if( ioctlCANID_GET_STATUS((handle_t)pBuf, 0 ) & CANID_EMPTY )
                break;
        }
        if( pBuf->flags & CAN_BUF_ERROR )
        {
            CANErrno = CAN_ERR_LOST;
            return (-1);
        }
    }

    return size;
}

/*******************************************************************************
*
* NAME: ioctlCANID_GET_STATUS
*
* DESCRIPTION: Driver i/o control procedure - Gets status of message buffer
*              
********************************************************************************
* PARAMETERS:       file descriptor, command and parameters list
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:   returned status value can be one of the following:
*                           CANID_EMPTY - no messages in the message buffer
*                           CANID_FULL - a message is in the message buffer
*                           CANID_OVERFLOW - a READ message buffer queue is full
*                                           and received message was lost 
*
* DEPENDENCIES:     None
*******************************************************************************/
EXPORT UWord16 ioctlCANID_GET_STATUS(handle_t FileDesc, void * pParams )      
{
	can_sReadBuffer * pHandle = (can_sReadBuffer *)FileDesc;
	
    if ( ((can_sReadBuffer*) pHandle)->flags & CAN_BUF_READ_HANDLE )
    {
        register UWord16 status = 0;
        can_sReadBuffer* pBuf = (can_sReadBuffer*) pHandle;

        if( ((can_sReadBuffer*) pHandle)->flags & CAN_BUF_OVERFLOW )
            status = CANID_OVERFLOW;
        else if( ((can_sReadBuffer*) pHandle)->flags & CAN_BUF_FULL )
            status = CANID_FULL;
        if(  ((can_sReadBuffer*) pHandle)->data[0].length == 0 )
            status  = CANID_EMPTY;
        return status;
    }
    else
    {
		register idx;

        idx =((can_sWriteBuffer*) pHandle)->flags & ( CAN_IDX_BUF0 | CAN_IDX_BUF1 | CAN_IDX_BUF2 );

        if( idx & ~periphMemRead(&ArchIO.CAN.TxFlagReg)  )
        {   
            if( ((can_sWriteBuffer*) pHandle)->canID == 
                    GetCANID((volatile UWord16 *)(&ArchIO.CAN.TxBuffer[ idx >> 1 ])) )  /* additional check */
                return CANID_FULL;
            else 
                return CANID_EMPTY; 
        }
        else
        {   /* buffer ready */
            return CANID_EMPTY;
        }
    }

}

/*******************************************************************************
*
* NAME: ioctlCAN_RESET
*
* DESCRIPTION: Driver i/o control procedure - Reset CAN Driver
*              
********************************************************************************
* PARAMETERS:       file descriptor,  parameters list
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:  
*
* DEPENDENCIES:     None
*******************************************************************************/
EXPORT UWord16 ioctlCAN_RESET(handle_t FileDesc, void * pParams )
{
    CANInit();
    return 0;
}


/*******************************************************************************
*
* NAME: ioctlCAN_SET_SLEEP
*
* DESCRIPTION: Driver i/o control procedure - Sets CAN into SLEEP mode
*              
********************************************************************************
* PARAMETERS:       file descriptor, command and parameters list
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:   
*
* DEPENDENCIES:     None
*******************************************************************************/
EXPORT UWord16 ioctlCAN_SET_SLEEP(handle_t FileDesc, void * pParams )
{
    periphBitSet(CANSLPRQ, &ArchIO.CAN.Control0Reg ); /* goto sleep mode */
    return 0;
}

/*******************************************************************************
*
* NAME: ioctlCAN_SET_WAKEUP
*
* DESCRIPTION: Driver i/o control procedure - Wakeups CAN
*              
********************************************************************************
* PARAMETERS:       file descriptor, command and parameters list
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:   
*
* DEPENDENCIES:     None
*******************************************************************************/
EXPORT UWord16 ioctlCAN_SET_WAKEUP(handle_t FileDesc, void * pParams )
{
    if( periphBitTest(CANSLPRQ, &ArchIO.CAN.Control0Reg )  )
        periphBitClear(CANSLPRQ, &ArchIO.CAN.Control0Reg );
    return 0;
}

/*******************************************************************************
*
* NAME: ioctlCAN_GET_STATUS
*
* DESCRIPTION: Driver i/o control procedure - Gets status of CAN control registers
*              
********************************************************************************
* PARAMETERS:       file descriptor, command and parameters list
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:   returned status value is composed of CANCTL0 + CANRFLG
*
* DEPENDENCIES:     None
*******************************************************************************/
EXPORT UWord16 ioctlCAN_GET_STATUS(handle_t FileDesc, void * pParams )
{
 /* *(UWord16*)pParams =  */
    return  ( periphMemRead(&ArchIO.CAN.Control0Reg ) << 8 )
           | periphMemRead( &ArchIO.CAN.RxFlagReg );

}

/*******************************************************************************
*
* NAME:             Close
*
* DESCRIPTION:      Driver close procedure
*              
********************************************************************************
* PARAMETERS:       file descriptor
*
* SIDE EFFECTS:     filters is staying open
*
* DESIGNER NOTES:   close corresponding message buffer
*
* DEPENDENCIES:     None
*******************************************************************************/
int canClose(handle_t FileDesc)
{
	can_sReadBuffer * pHandle = (can_sReadBuffer *)FileDesc;

    if ( ((can_sReadBuffer*) pHandle)->flags & CAN_BUF_READ_HANDLE )
    {
        if( (char*)pHandle == ((char*)pLastRead) - sIState.RxSize )
        {
            ((can_sReadBuffer*) pHandle)->flags = 0;
            ((can_sReadBuffer*) pHandle)->data[0].length = 0;
			pLastRead = (can_sReadBuffer*)(((char*)pLastRead)-sIState.RxSize);
        }
        else
        {
            return -1;
        }
    }
    else 
    {
        if( (can_sWriteBuffer*)pHandle == pLastWrite - 1 ) /* correct */
        {
            ((can_sWriteBuffer*) pHandle)->flags = 0;
            pLastWrite--;
        }
        else
        {
            return -1;
        }
    }
    return 0;
}

/*******************************************************************************
*
* NAME:             CANDevCreat
*
* DESCRIPTION:      Create CAN driver device installation
*              
********************************************************************************
* PARAMETERS:       A pointer to the initialization structure of type 
*                   can_sInitialState
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:   The MSCAN interrupts installation and device initialization.
*
* DEPENDENCIES:     None
*******************************************************************************/
UWord16 CANDevCreat(const can_sInitialState* pInitialState)
{
/*  UWord16 driverNumber;	*/
    sIState = *pInitialState; /* need to copy user driver configuration */

/*    driverNumber = ioDrvInstall(canOpen); */
    
#if 0    /* install ISR - obsolete */
    archInstallISR(&(pArchInterrupts->MscanReceiverReady), CANReceiveISR);
    archInstallISR(&(pArchInterrupts->MscanTransmiterReady), CANSendISR);
    archInstallISR(&(pArchInterrupts->MscanError), CANErrorISR);
    archInstallISR(&(pArchInterrupts->MscanWakeup), CANWakeUpISR);
#endif /* 0 */

    pLastRead  = sIState.pRxBuffer;
    pLastWrite = sIState.pTxBuffer;

#if 0    /* install raw callback - done statically */
    if (sIState.pRawCallBack != NULL)
        CANRcvPrepare = sIState.pRawCallBack;
    else
        CANRcvPrepare = CANStdRcvPrepare;
#endif /* 0 */

    actFilterLimit = 0;
    if (sIState.code <= CANIDMASK)
    {   /* set custom filter */
        sCANFilterMirror[0].code = (CANADDRESSTYPE)(sIState.code);
        sCANFilterMirror[0].mask = (CANADDRESSTYPE)(sIState.mask);
        actFilterLimit = 1;
    }

    CANInit();

    return 0;

}
#endif /* defined(DSP56838EVM) */
